import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from requests import Session
from pydactyl.api import base


class BaseTests(unittest.TestCase):

    def setUp(self):
        self.api = base.PterodactylAPI(url='https://dummy.com',
                                       api_key='dummy_key')

    def test_init(self):
        self.assertEqual('dummy_key', self.api._api_key)
        self.assertEqual('https://dummy.com/api', self.api._url)

    def test_get_headers(self):
        expected = {
            'Authorization': 'Bearer dummy_key',
            'Accept': 'application/json',
            'Content-Type': 'application/json',
        }
        self.assertDictEqual(expected, self.api._get_headers())

    def test_url_join(self):
        self.assertEqual('www.test.com/path', base.url_join(
            'www.test.com', 'path'))
        self.assertEqual('test.com/path/to/thing',
                         base.url_join('test.com', 'path', 'to', 'thing'))
        self.assertEqual('https://asdf.com/api/other/things',
                         base.url_join('https://asdf.com', '/api/',
                                       'other/things'))

    def test_parse_response_with_detail(self):
        expected = {'object': 'server', 'attributes': {'id': 12}}
        self.assertEqual(expected, base.parse_response(expected, detail=True))

    def test_parse_response_without_detail(self):
        expected = {'id': 12}
        response = {'object': 'server', 'attributes': {'id': 12}}
        self.assertEqual(expected, base.parse_response(response, detail=False))

    def test_parse_response_list_object_with_detail(self):
        expected = {'object': 'list',
                    'data': [{'object': 'server', 'attributes': {'id': 5}},
                             {'object': 'server', 'attributes': {'id': 6}}]}
        self.assertEqual(expected, base.parse_response(expected, detail=True))

    def test_parse_response_list_object_without_detail(self):
        expected = [{'id': 5}, {'id': 6}]
        response = {'object': 'list',
                    'data': [{'object': 'server', 'attributes': {'id': 5}},
                             {'object': 'server', 'attributes': {'id': 6}}]}
        self.assertEqual(expected, base.parse_response(response, detail=False))

    @mock.patch.object(Session, 'get')
    def test_valid_api_get_request(self, mock_request):
        expected = {
            'params': None,
            'headers': self.api._get_headers(),
        }
        self.api._api_request(endpoint='nomorecoffee')
        mock_request.assert_called_with('https://dummy.com/api/nomorecoffee',
                                        **expected)

    @mock.patch.object(Session, 'post')
    def test_valid_api_post_request(self, mock_request):
        data = {'test': 'data'}
        expected = {
            'params': None,
            'headers': self.api._get_headers(),
        }
        self.api._api_request(endpoint='foo', mode='POST', data=data)
        mock_request.assert_called_with('https://dummy.com/api/foo', json=data,
                                        **expected)

    @mock.patch.object(Session, 'patch')
    def test_valid_api_patch_request(self, mock_request):
        data = {'patch': 'me'}
        expected = {
            'params': None,
            'headers': self.api._get_headers(),
        }
        self.api._api_request(endpoint='bar', mode='PATCH', data=data)
        mock_request.assert_called_with('https://dummy.com/api/bar', json=data,
                                        **expected)

    @mock.patch.object(Session, 'delete')
    def test_valid_api_delete_request(self, mock_request):
        expected = {
            'params': None,
            'headers': self.api._get_headers(),
        }
        self.api._api_request(endpoint='havecoffee', mode='DELETE')
        mock_request.assert_called_with('https://dummy.com/api/havecoffee',
                                        **expected)



from pydactyl.api import base
from pydactyl.api.base import PterodactylAPI
from pydactyl.exceptions import BadRequestError
from pydactyl.responses import PaginatedResponse

class User(PterodactylAPI):
    """Class for interacting with the Pterdactyl Client API."""

    def list_users(self, search=None, detail=True):
        """List all users.

        Args:
            search(str): Filter user list by search term.
            detail(bool): If True includes created and updated timestamps.
        """
        params = {}
        if search is not None:
            params = {'search': search}
        response = self._api_request(endpoint='application/users',
                                     params=params)
        return PaginatedResponse(self, 'application/users', response)

    def get_user_info(self, user_id=None, external_id=None, detail=True):
        """List detailed user information for specified user_id.

        Args:
            user_id(int): Pterodactyl user ID
            external_id(int): User ID from external system like WHMCS
            detail(bool): If True includes created and updated timestamps.
        """
        if not user_id and not external_id:
            raise BadRequestError('Must specify either user_id or external_id.')
        if user_id and external_id:
            raise BadRequestError('Specify either user_id or external_id, '
                                  'not both.')

        if user_id:
            endpoint = 'application/users/%s' % user_id
        else:
            endpoint = 'application/users/external/%s' % external_id

        response = self._api_request(endpoint=endpoint)
        return base.parse_response(response, detail=detail)

    def create_user(self, username, email, first_name, last_name,
                    external_id=None, password=None, root_admin=False,
                    language='en'):
        """Creates a primary user on the Pterodactyl panel.

        Args:
            username(str): Username, must be unique, required.
            email(str): Full email, required.
            first_name(str): User's first name, required.
            last_name(str): User's last name, required.
            external_id(str): User ID from external system like WHMCS.
            password(str): Optionally specify a password.  If not specified
                    user will receive an email to setup their password.
            root_admin(bool): Whether the account is an admin.
            language(str): Used to specify the default language for an account.
        """
        data = {'username': username,
                'email': email,
                'first_name': first_name,
                'last_name': last_name,
                'external_id': external_id,
                'password': password,
                'root_admin': root_admin,
                'language': language,
                }
        response = self._api_request(endpoint='application/users',
                                     mode='POST', data=data)
        return base.parse_response(response, detail=True)

    def edit_user(self, user_id, username, email, first_name, last_name,
                  external_id=None, password=None, root_admin=False,
                  language='en'):
        """Edits an existing user on the Pterodactyl panel.

        The user_id is used to find the existing user.  All other parameters
        will be updated on that user_id.

        Args:
            user_id(int): Pterodactyl User ID for the user to update.
            username(str): Username, must be unique, required.
            email(str): Full email, required.
            first_name(str): User's first name, required.
            last_name(str): User's last name, required.
            external_id(str): User ID from external system like WHMCS.
            password(str): Optionally specify a password.  If not specified
                    user will receive an email to setup their password.
            root_admin(bool): Whether the account is an admin.
            language(str): Used to specify the default language for an account.
        """
        data = {'username': username,
                'email': email,
                'first_name': first_name,
                'last_name': last_name,
                'external_id': external_id,
                'password': password,
                'root_admin': root_admin,
                'language': language,
                }
        response = self._api_request(endpoint='application/users/%s' % user_id,
                                     mode='PATCH', data=data)
        return base.parse_response(response, detail=True)

    def delete_user(self, user_id=None, external_id=None, detail=True):
        """Deletes the specified user.

        Will look up by external_id if provided.  This action cannot be
        reversed so use with care.

        Args:
            user_id(int): Pterodactyl user ID
            external_id(int): User ID from external system like WHMCS
            detail(bool): If True includes created and updated timestamps.
        """
        if not user_id and not external_id:
            raise BadRequestError('Must specify either user_id or external_id.')
        if user_id and external_id:
            raise BadRequestError('Specify either user_id or external_id, '
                                  'not both.')

        if external_id:
            response = self._api_request(
                endpoint='application/users/external/%s' % external_id)
            user_id = response['attributes']['id']

        response = self._api_request(
            endpoint='application/users/%s' % user_id, mode='DELETE')
        return base.parse_response(response, detail=detail)



from pydactyl.api.base import PterodactylAPI
from pydactyl.constants import USE_SSL
from pydactyl.exceptions import BadRequestError
from pydactyl.responses import PaginatedResponse


class Nodes(PterodactylAPI):
    """Class for interacting with the Pterdactyl Nodes API."""

    def list_nodes(self):
        """List all nodes."""
        response = self._api_request(endpoint='application/nodes')
        return response

    def get_node_info(self, node_id):
        """Get detailed info for the specified node.

        Args:
            node_id(int): Pterodactyl Node ID.
        """
        response = self._api_request(endpoint='application/nodes/%s' % node_id)
        return response

    def create_node(self, name, description, location_id, fqdn, memory, disk,
                    memory_overallocate=0,
                    disk_overallocate=0, use_ssl=True, behind_proxy=False,
                    daemon_base='/srv/daemon-data',
                    daemon_sftp=2022, daemon_listen=8080, upload_size=100,
                    public=True, maintenance_mode=False):
        """Creates a new node.

        Args:
            name(str): Node Name, 1-100 characters, valid characters: a-zA-Z0-9_.-[Space]
            description(str): A long description of the node.
            location_id(int): A valid Location ID
            fqdn(str): Domain name used to connect to the daemon.  Alternatively an IP address if not using SSL.
            memory(int): Memory in MB that is available to the daemon for allocation to servers.
            memory_overallocate(int): Percentage of memory that can be overallocated, e.g. 150
            disk(int): Disk space in MB that is available to the daemon for allocation to servers.
            disk_overallocate(int): Percentage of disk space that can be overallocated, e.g. 150
            use_ssl(bool): True to enable SSL, false for insecure HTTP
            behind_proxy(bool): Set to True if running behind a proxy like CloudFlare.  Skips certificate check on boot.
            daemon_base(str): Directory where server files will be stored.
            daemon_sftp(int): Port used by daemon for SFTP.
            daemon_listen(int): Port used by the daemon.
            public(bool): Set to False to prevent servers from being created on this node.
            maintenance_mode(bool): Set to True to disable the node or something.
        """
        data = locals()
        del data['self']
        del data['use_ssl']
        data['scheme'] = USE_SSL[use_ssl]

        response = self._api_request(endpoint='application/nodes',
                                     mode='POST', data=data)
        return response

    def edit_node(self, node_id, shortcode=None, description=None):
        """Modify an existing node.

        Args:
            node_id(int): Pterodactyl Node ID.
            shortcode(str): Short identifier between 1 and 60 characters, e.g. us.nyc.lvl3
            description(str): A long description of the node.  Max 255 characters.
        """
        if not shortcode and not description:
            raise BadRequestError(
                'Must specify either shortcode or description for edit_node.')

        data = {}
        if shortcode:
            data['shortcode'] = shortcode
        if description:
            data['description'] = description

        response = self._api_request(endpoint='application/nodes/%s' %
                                              node_id, mode='PATCH', data=data)
        return response

    def delete_node(self, node_id):
        """Delete an existing node.

        Args:
            node_id(int): Pterodactyl Node ID.
        """
        response = self._api_request(endpoint='application/nodes/%s' %
                                              node_id, mode='DELETE')
        return response

    def list_node_allocations(self, node_id):
        """Retrieves all allocations for a specified node.

        Args:
            node_id(int): Pterodactyl Node ID.

        Returns:
            obj: Iterable response that fetches pages as required.
        """
        endpoint = 'application/nodes/%s/allocations' % node_id
        response = self._api_request(endpoint=endpoint)
        return PaginatedResponse(self, endpoint, response)

    def create_allocations(self, node_id, ip, ports, alias=None):
        """Create one or more allocations.

        Args:
            node_id(int): Pterodactyl Node ID.
            ip(str): The IP address to create the allocation on.
            ports(iter): List of strings representing strings.  Can use
                ranges as supported by Pterodactyl, e.g. ["4000", "4003-4005"]
            alias(str): Optional IP alias.  Used if you want to display a
                different IP address to Panel users, for example to display the
                external IP when behind a NAT.
        """
        data = {'ip': ip, 'ports': ports}
        if alias:
            data['alias'] = alias
        response = self._api_request(
            endpoint='application/nodes/%s/allocations' % node_id,
            mode='POST', data=data, json=False)
        return response

    def delete_allocation(self, node_id, allocation_id):
        """Deletes the specified allocation on the specified node.

        Args:
            node_id(int): Pterodactyl Node ID.
            allocation_id(int): Pterodactyl Allocation ID.  This is the
                internal ID assigned to the allocation, not the port number.
        """
        response = self._api_request(
            endpoint='application/nodes/%s/allocations/%s' % (node_id,
                                                              allocation_id),
            mode='DELETE', json=False)
        return response



"""Pydactyl constants."""

__version__ = '0.1.12'

USER_AGENT = 'Pydactyl/' + __version__
POWER_SIGNALS = ('start', 'stop', 'restart', 'kill')
USE_SSL = {True: 'https', False: 'http'}
REQUEST_TYPES = ('GET', 'POST', 'PATCH', 'DELETE')



from pydactyl.api import Client
from pydactyl.api import Locations
from pydactyl.api import Nests
from pydactyl.api import Nodes
from pydactyl.api import Servers
from pydactyl.api import User
from pydactyl.exceptions import ClientConfigError


class PterodactylClient(object):
    """Provides a simplified interface to the Pterodactyl Panel API.

    Instances of this class allow interaction with the Pterodactyl Panel API.
    """

    def __init__(self, url=None, api_key=None):
        """Initialize a Pterodactyl class instance.

        Args:
            url(str): The base URL of the panel to connect to.
            api_key(str): Pterodactyl Panel API key.
        """
        if not url:
            raise ClientConfigError(
                'You must specify the hostname of a Pterodactyl instance.')

        if not api_key:
            raise ClientConfigError(
                'You must specify a Pterodactyl API key to authenticate.')

        self._url = url
        self._api_key = api_key

        self._client = None
        self._locations = None
        self._nests = None
        self._nodes = None
        self._servers = None
        self._user = None

    @property
    def client(self):
        self._client = Client(self._url, self._api_key)
        return self._client

    @property
    def locations(self):
        self._locations = Locations(self._url, self._api_key)
        return self._locations

    @property
    def nests(self):
        self._nests = Nests(self._url, self._api_key)
        return self._nests

    @property
    def nodes(self):
        self._nodes = Nodes(self._url, self._api_key)
        return self._nodes

    @property
    def servers(self):
        self._servers = Servers(self._url, self._api_key)
        return self._servers

    @property
    def user(self):
        self._user = User(self._url, self._api_key)
        return self._user



import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from pydactyl import PterodactylClient
from pydactyl.exceptions import BadRequestError


class UserTests(unittest.TestCase):

    def setUp(self):
        self.client = PterodactylClient(url='dummy', api_key='dummy')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_users(self, mock_api):
        expected = {
            'endpoint': 'application/users',
            'params': {},
        }
        self.client.user.list_users()
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_list_users_with_search(self, mock_api):
        expected = {
            'endpoint': 'application/users',
            'params': {'search': 'best@test.com'}
        }
        self.client.user.list_users(search='best@test.com')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_user_info_by_external_id(self, mock_api):
        expected = {
            'endpoint': 'application/users/external/11',
        }
        self.client.user.get_user_info(external_id=11)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_user_info_by_user_id(self, mock_api):
        expected = {
            'endpoint': 'application/users/22',
        }
        self.client.user.get_user_info(user_id=22)
        mock_api.assert_called_with(**expected)

    def test_get_user_info_raises_with_no_id(self):
        with self.assertRaises(BadRequestError):
            self.client.user.get_user_info()

    def test_get_user_info_raises_with_both_ids(self):
        with self.assertRaises(BadRequestError):
            self.client.user.get_user_info(user_id=1, external_id=2)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_user_with_user_id(self, mock_api):
        expected = {
            'endpoint': 'application/users/77',
            'mode': 'DELETE',
        }
        self.client.user.delete_user(user_id=77)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_delete_user_with_external_id(self, mock_api):
        mock_api.return_value = {'attributes': {'id': 88}}
        expected = [mock.call(endpoint='application/users/external/11'),
                    mock.call(endpoint='application/users/88', mode='DELETE')]
        self.client.user.delete_user(external_id=11)
        mock_api.assert_has_calls(expected)

    def test_delete_user_raises_with_no_id(self):
        with self.assertRaises(BadRequestError):
            self.client.user.delete_user()

    def test_delete_user_raises_with_both_ids(self):
        with self.assertRaises(BadRequestError):
            self.client.user.delete_user(user_id=1, external_id=2)


if __name__ == '__main__':
    unittest.main()



from pydactyl.api.base import PterodactylAPI
from pydactyl.exceptions import BadRequestError


class Locations(PterodactylAPI):
    """Class for interacting with the Pterdactyl Locations API."""

    def list_locations(self):
        """List all locations."""
        response = self._api_request(endpoint='application/locations')
        return response

    def get_location_info(self, location_id):
        """Get detailed info for the specified location.

        Args:
            location_id(int): Pterodactyl Location ID.
        """
        response = self._api_request(
            endpoint='application/locations/%s' % location_id)
        return response

    def create_location(self, shortcode, description):
        """Creates a new location.

        Args:
            shortcode(str): Short identifier between 1 and 60 characters, e.g. us.nyc.lvl3
            description(str): A long description of the location.  Max 255 characters.
        """
        data = {'shortcode': shortcode, 'description': description}
        response = self._api_request(endpoint='application/locations',
                                     mode='POST', data=data)
        return response

    def edit_location(self, location_id, shortcode=None, description=None):
        """Modify an existing location.

        Args:
            location_id(int): Pterodactyl Location ID.
            shortcode(str): Short identifier between 1 and 60 characters, e.g. us.nyc.lvl3
            description(str): A long description of the location.  Max 255 characters.
        """
        if not shortcode and not description:
            raise BadRequestError(
                'Must specify either shortcode or description for edit_location.')

        data = {}
        if shortcode:
            data['shortcode'] = shortcode
        if description:
            data['description'] = description

        response = self._api_request(
            endpoint='application/locations/%s' % location_id, mode='PATCH',
            data=data)
        return response

    def delete_location(self, location_id):
        """Delete an existing location.

        Args:
            location_id(int): Pterodactyl Location ID.
        """
        response = self._api_request(
            endpoint='application/locations/%s' % location_id, mode='DELETE')
        return response



import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from pydactyl import PterodactylClient
from pydactyl.exceptions import BadRequestError


class ClientTests(unittest.TestCase):

    def setUp(self):
        self.client = PterodactylClient(url='dummy', api_key='dummy')

    def test_invalid_power_action_raises_exception(self):
        with self.assertRaises(BadRequestError):
            self.client.client.send_power_action(1, 'BADSIGNAL')

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_server(self, mock_api):
        expected = {
            'endpoint': 'client/servers/11',
        }
        self.client.client.get_server(11)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_get_server_utilization(self, mock_api):
        expected = {
            'endpoint': 'client/servers/22/utilization',
        }
        self.client.client.get_server_utilization(22)
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_send_console_command(self, mock_api):
        expected = {
            'endpoint': 'client/servers/33/command',
            'mode': 'POST',
            'data': {'command': 'say Test Command'},
            'json': False,
        }
        self.client.client.send_console_command(33, 'say Test Command')
        mock_api.assert_called_with(**expected)

    @mock.patch('pydactyl.api.base.PterodactylAPI._api_request')
    def test_valid_power_action_valid_request(self, mock_api):
        expected = {
            'endpoint': 'client/servers/1/power',
            'mode': 'POST',
            'data': {'signal': 'start'},
            'json': False,
        }
        self.client.client.send_power_action(1, 'start')
        mock_api.assert_called_with(**expected)


if __name__ == '__main__':
    unittest.main()



import os
import re
import sys

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
with open("pydactyl/constants.py") as fh:
    VERSION = re.search('__version__ = \'([^\']+)\'', fh.read()).group(1)

# 'setup.py publish' shortcut.
if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()


setuptools.setup(
    name="py-dactyl",
    version=VERSION,
    author="Ryan Kubiak",
    author_email="iamkubi@gmail.com",
    description="An easy to use Python wrapper for the Pterodactyl Panel API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/iamkubi/pydactyl",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.4',
    install_requires=[
        "requests >=2.21.0",
    ],
    tests_require=[
        "pytest >=3",
        "pytest-cov",
    ],
    project_urls={
        "Documentation": "https://pydactyl.readthedocs.io/",
        "Source": "https://github.com/iamkubi/pydactyl",
    }
)



"""
Pterodactyl API Wrapper.

Pterodactyl is an open source game control panel.  Pydactyl is a Python package
that provides a simplified interface to the API.
"""

from .client import PterodactylClient  # noqa
from .constants import __version__  # noqa
